#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "globals.h"

int st_levels = 30;
int st_level_depth = 1;
int random_texture_width = 128;
int random_texture_height = 128;

void
load_texture (GtkButton * button, gpointer user_data)
{
  GtkWidget *texture;
  GtkWidget *viewport;
  GtkWidget *file_selector = (GtkWidget *) button;
  const gchar *selected_filename;

  selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
  gtk_widget_destroy ((GtkWidget *) button);
  viewport = lookup_widget (main_win, "texture_viewport");
  texture = lookup_widget (viewport, "texture");
  gtk_container_remove (GTK_CONTAINER (viewport), texture);
  gtk_widget_destroy (texture);
  texture = create_pixmap (main_win, selected_filename);
  gtk_widget_show (texture);
  gtk_container_add (GTK_CONTAINER (viewport), texture);
  g_object_set_data_full (G_OBJECT (main_win), "texture", gtk_widget_ref (texture),
			  (GDestroyNotify) gtk_widget_unref);
}

void
on_load_texture_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *file_selector;
  file_selector = create_texturefileselection ();
  gtk_widget_show (file_selector);
}

void
on_texture_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *file_selector;
  file_selector = create_texturefileselection ();
  gtk_widget_show (file_selector);
}

void
load_image (GtkButton * button, gpointer user_data)
{
  GtkWidget *image;
  GtkWidget *viewport;
  GtkWidget *file_selector = (GtkWidget *) button;
  const gchar *selected_filename;

  selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
  gtk_widget_destroy ((GtkWidget *) button);
  viewport = lookup_widget (main_win, "image_viewport");
  image = lookup_widget (viewport, "image");
  gtk_container_remove (GTK_CONTAINER (viewport), image);
  gtk_widget_destroy (image);
  image = create_pixmap (main_win, selected_filename);
  gtk_widget_show (image);
  gtk_container_add (GTK_CONTAINER (viewport), image);
  g_object_set_data_full (G_OBJECT (main_win), "image", gtk_widget_ref (image),
			  (GDestroyNotify) gtk_widget_unref);
}

void
on_load_image_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *file_selector;
  file_selector = create_imagefileselection ();
  gtk_widget_show (file_selector);
}

void
on_image_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *file_selector;
  file_selector = create_imagefileselection ();
  gtk_widget_show (file_selector);
}


void
on_quit_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  gtk_main_quit ();
}


void
on_about_dlg_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  about_dlg = create_about_dlg ();
  gtk_widget_show (about_dlg);
}


void
on_main_win_destroy (GtkObject * object, gpointer user_data)
{
  gtk_main_quit ();
}

void
on_make_button_clicked (GtkButton * button, gpointer user_data)
{
  int i, j, mid, s, ibs, bbs, cs, bits, im_alpha, bk_alpha;
  int ih, iw, iiw, th, tw, tw2, sd, so;
  GdkPixbuf *im_buf, *tx_buf, *st_buf;
  GtkWidget *texture, *image, *stereogram, *viewport;
  guchar *im_px, *tx_px, *st_px;

  image = lookup_widget (main_win, "image");
  if (gtk_image_get_storage_type (GTK_IMAGE (image)) == GTK_IMAGE_EMPTY)
    return;
  im_buf = gtk_image_get_pixbuf (GTK_IMAGE (image));
  ih = gdk_pixbuf_get_height (im_buf);
  iw = gdk_pixbuf_get_width (im_buf);
  bits = gdk_pixbuf_get_bits_per_sample (im_buf);
  im_alpha = gdk_pixbuf_get_has_alpha (im_buf);
  im_px = gdk_pixbuf_get_pixels (im_buf);
  ibs = 3 + im_alpha;

  texture = lookup_widget (main_win, "texture");
  if (gtk_image_get_storage_type (GTK_IMAGE (texture)) == GTK_IMAGE_EMPTY)
    return;
  tx_buf = gtk_image_get_pixbuf (GTK_IMAGE (texture));
  th = gdk_pixbuf_get_height (tx_buf);
  tw = gdk_pixbuf_get_width (tx_buf);
  bits = gdk_pixbuf_get_bits_per_sample (tx_buf);
  bk_alpha = gdk_pixbuf_get_has_alpha (tx_buf);
  tx_px = gdk_pixbuf_get_pixels (tx_buf);
  bbs = 3 + bk_alpha;
  st_buf = gdk_pixbuf_copy (im_buf);
  st_px = gdk_pixbuf_get_pixels (st_buf);
  mid = iw / 2;
  ibs = 3 + im_alpha;
  tw2 = tw / 2;
  sd = 256 / st_levels;
  so = st_levels * st_level_depth;
  for (i = 0; i < ih; i++)
    {
      int x1, x2, offset;
      offset = 0;
      for (j = mid - tw2; j <= mid + tw2; j++)
	{
	  x1 = ibs * (i * iw + j);
	  x2 = bbs * ((i % th) * tw + offset);
	  st_px[x1] = tx_px[x2];
	  st_px[x1 + 1] = tx_px[x2 + 1];
	  st_px[x1 + 2] = tx_px[x2 + 2];
	  if (im_alpha == 1)
	    st_px[x1 + 3] = 255;
	  offset = (offset + 1) % tw;
	}
    }
  for (i = 0; i < ih; i++)
    {
      int x0, x1, x2, offset, start;
      for (j = mid - tw2 + so - im_px[ibs * (i * iw + mid - 1)] / sd * st_level_depth; j >= 0; j--)
	{
	  x0 = i * iw + j;
	  x1 = ibs * x0;
	  offset = tw - so + im_px[ibs * (x0 + tw2)] / sd * st_level_depth;
	  x2 = ibs * (x0 + offset);
	  st_px[x1] = st_px[x2];
	  st_px[x1 + 1] = st_px[x2 + 1];
	  st_px[x1 + 2] = st_px[x2 + 2];
	}
      for (j = mid + tw2 - so + im_px[ibs * (i * iw + mid + 1)] / sd * st_level_depth; j < iw; j++)
	{
	  x0 = i * iw + j;
	  x1 = ibs * x0;
	  offset = tw - so + im_px[ibs * (x0 - tw2)] / sd * st_level_depth;
	  x2 = ibs * (x0 - offset);
	  st_px[x1] = st_px[x2];
	  st_px[x1 + 1] = st_px[x2 + 1];
	  st_px[x1 + 2] = st_px[x2 + 2];
	}
    }
  viewport = lookup_widget (main_win, "stereogram_viewport");
  stereogram = lookup_widget (main_win, "stereogram");
  gtk_container_remove (GTK_CONTAINER (viewport), stereogram);
  gtk_widget_destroy (stereogram);
  stereogram = gtk_image_new_from_pixbuf (st_buf);
  gtk_container_add (GTK_CONTAINER (viewport), stereogram);
  gtk_widget_show (stereogram);
  g_object_set_data_full (G_OBJECT (main_win), "stereogram", gtk_widget_ref (stereogram),
			  (GDestroyNotify) gtk_widget_unref);
}


void
on_save_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *file_selector;
  file_selector = create_savefileselection ();
  gtk_widget_show (file_selector);
}


void
on_save_stereogram_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *file_selector;
  file_selector = create_savefileselection ();
  gtk_widget_show (file_selector);
}

void
save_stereogram (GtkButton * button, gpointer user_data)
{
  GdkPixbuf *st_buf;
  GtkWidget *stereogram;
  GtkWidget *file_selector = (GtkWidget *) button;
  const gchar *selected_filename;
  GError *error = NULL;

  stereogram = lookup_widget (main_win, "stereogram");
  if (gtk_image_get_storage_type (GTK_IMAGE (stereogram)) == GTK_IMAGE_EMPTY)
    return;
  st_buf = gtk_image_get_pixbuf (GTK_IMAGE (stereogram));
  selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
  gtk_widget_destroy ((GtkWidget *) button);
  gdk_pixbuf_save (st_buf, (const char *) selected_filename, "jpeg", &error, "quality", "100",
		   NULL);
}

void
on_randomtexture_clicked (GtkButton * button, gpointer user_data)
{
  GdkPixbuf *tx_buf;
  guchar *tx_px;
  int i;
  GtkWidget *viewport;
  GtkWidget *texture;
  tx_buf =
    gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, random_texture_width, random_texture_height);
  tx_px = gdk_pixbuf_get_pixels (tx_buf);
  for (i = 0; i < random_texture_width * random_texture_height * 3; i++)
    tx_px[i] = rand () % 256;
  viewport = lookup_widget (main_win, "texture_viewport");
  texture = lookup_widget (viewport, "texture");
  gtk_container_remove (GTK_CONTAINER (viewport), texture);
  gtk_widget_destroy (texture);
  texture = gtk_image_new_from_pixbuf (tx_buf);
  gtk_container_add (GTK_CONTAINER (viewport), texture);
  gtk_widget_show (texture);
  g_object_set_data_full (G_OBJECT (main_win), "texture", gtk_widget_ref (texture),
			  (GDestroyNotify) gtk_widget_unref);
}

void
on_st_levels_spinbutton_value_changed (GtkSpinButton * spinbutton, gpointer user_data)
{
  st_levels = gtk_spin_button_get_value_as_int (spinbutton);
}


void
on_level_depth_spinbutton_value_changed (GtkSpinButton * spinbutton, gpointer user_data)
{
  st_level_depth = gtk_spin_button_get_value_as_int (spinbutton);
}


void
  on_random_texture_width_spinbutton_value_changed (GtkSpinButton * spinbutton, gpointer user_data)
{
  random_texture_width = gtk_spin_button_get_value_as_int (spinbutton);
}


void
  on_random_texture_height_spinbutton_value_changed (GtkSpinButton * spinbutton, gpointer user_data)
{
  random_texture_height = gtk_spin_button_get_value_as_int (spinbutton);
}
